@@ -23,6 +23,7 @@ along with this program; if not, see http://www.gnu.org/licenses/
 #include "postwidget.h"
 #include "choqoktools.h"
 #include "textbrowser.h"
+#include "urlutils.h"
 #include <qboxlayout.h>
 #include <KLocale>
 #include <KPushButton>
@@ -66,8 +67,11 @@ class PostWidget::Private
         //BEGIN UI contents:
         QString mSign;
         QString mContent;
+        QString mProfileImage;
         QString mImage;
+        QString imageUrl;
         QString dir;
+	QPixmap originalImage;
         //END UI contents;
 
         QStringList detectedUrls;
@@ -75,39 +79,16 @@ class PostWidget::Private
         TimelineWidget *timeline;
 };
 
+const QString PostWidget::ownText ("<table width=\"100%\" ><tr><td width=\"90%\" dir=\"%4\">%2</td><td  rowspan=\"2\" align=\"right\">%1</td></tr><tr>%5</tr><tr><td colspan=\"2\"  style=\"font-size:small;\" dir=\"ltr\" align=\"right\" width=\"100%\" valign=\"bottom\">%3</td></tr></table>");
 
-const QString PostWidget::ownText ("<table width=\"100%\" ><tr><td dir=\"%4\"><p>%2</p></td><td width=\"5\"><!-- empty --></td><td width=\"48\" rowspan=\"2\" align=\"right\">%1</td></tr><tr><td style=\"font-size:small;\" dir=\"ltr\" align=\"right\" valign=\"bottom\">%3</td></tr></table>");
-
-const QString PostWidget::otherText ( "<table height=\"100%\" width=\"100%\"><tr><td rowspan=\"2\"\
-width=\"48\">%1</td><td width=\"5\"><!-- EMPTY HAHA --></td><td dir=\"%4\"><p>%2</p></td></tr><tr><td><!-- EMPTY HAHA --></td><td style=\"font-size:small;\" dir=\"ltr\" align=\"right\" width=\"100%\" valign=\"bottom\">%3</td></tr></table>");
+const QString PostWidget::otherText ( "<table height=\"100%\" width=\"100%\"><tr><td rowspan=\"2\" width=\"48\">%1</td><td width=\"5\"><!-- EMPTY HAHA --></td><td colspan=\"2\" dir=\"%4\"><p>%2</p></td></tr><tr><td></td>%5</tr><tr><td ><!-- empty --></td><td></td><td colspan=\"2\" style=\"font-size:small;\" dir=\"ltr\" align=\"right\" width=\"100%\" valign=\"bottom\">%3</td></tr></table>");
 
 const QString PostWidget::baseStyle ("KTextBrowser {border: 1px solid rgb(150,150,150);\
 border-radius:5px; color:%1; background-color:%2; %3}\
 KPushButton{border:0px}");
 
-const QString protocols = "((https?|ftps?)://)";
-const QString subdomains = "(([a-z0-9\\-_]{1,}\\.)?)";
-const QString auth = "(([a-z0-9\\-_]{1,})((:[\\S]{1,})?)@)";
-const QString domains = "(([a-z0-9\\-\\x0080-\\xFFFF_]){1,63}\\.)+";
-const QString port = "(:(6553[0-5]|655[0-2][0-9]|65[0-4][\\d]{2}|6[0-4][\\d]{3}|[1-5][\\d]{4}|[1-9][\\d]{0,3}))";
-const QString zone ("((a[cdefgilmnoqrstuwxz])|(b[abdefghijlmnorstvwyz])|(c[acdfghiklmnoruvxyz])|(d[ejkmoz])|(e[ceghrstu])|\
-(f[ijkmor])|(g[abdefghilmnpqrstuwy])|(h[kmnrtu])|(i[delmnoqrst])|(j[emop])|(k[eghimnprwyz])|(l[abcikrstuvy])|\
-(m[acdefghklmnopqrstuvwxyz])|(n[acefgilopruz])|(om)|(p[aefghklnrstwy])|(qa)|(r[eosuw])|(s[abcdeghijklmnortuvyz])|\
-(t[cdfghjkmnoprtvwz])|(u[agksyz])|(v[aceginu])|(w[fs])|(ye)|(z[amrw])\
-|(asia|com|info|net|org|biz|name|pro|aero|cat|coop|edu|jobs|mobi|museum|tel|travel|gov|int|mil|local|xxx)|(中国)|(公司)|(网络)|(صر)|(امارات)|(рф))");
-const QString ip = "(25[0-5]|[2][0-4][0-9]|[0-1]?[\\d]{1,2})(\\.(25[0-5]|[2][0-4][0-9]|[0-1]?[\\d]{1,2})){3}";
-const QString params = "(((\\/)[\\w:/\\?#\\[\\]@!\\$&\\(\\)\\*%\\+,;=\\._~\\x0080-\\xFFFF\\-\\|]{1,}|%[0-9a-f]{2})?)";
-const QString excludingCharacters = QString::fromLatin1("[^\\s`!()\\[\\]{};:'\".,<>?%1%2%3%4%5%6]") 
-                            .arg(QChar(0x00AB)).arg(QChar(0x00BB)).arg(QChar(0x201C)).arg(QChar(0x201D)).arg(QChar(0x2018)).arg(QChar(0x2019));
+const QString PostWidget::hrefTemplate("<a href='%1' title='%1' target='_blank'>%2</a>");
 
-const QRegExp PostWidget::mUrlRegExp("(((((" + protocols + auth + "?)?)" +
-                          subdomains +
-                          '(' + domains +
-                          zone + "(?!(\\w))))|(" + protocols + '(' + ip + ")+))" +
-                          '(' + port + "?)" + "((\\/)?)"  +
-                          params + ')' + excludingCharacters, Qt::CaseInsensitive);
-
-const QRegExp PostWidget::mEmailRegExp('^' + auth + subdomains + domains + zone);
 const QRegExp PostWidget::dirRegExp("(RT|RD)|(@([^\\s\\W]+))|(#([^\\s\\W]+))|(!([^\\s\\W]+))");
 
 QString PostWidget::readStyle;
@@ -129,6 +110,12 @@ PostWidget::PostWidget( Account* account, Choqok::Post* post, QWidget* parent/*
 
     d->timeline = qobject_cast<TimelineWidget*>(parent);
     d->mCurrentPost->owners++;
+
+    if(!d->mCurrentPost->media.isEmpty()) {
+        d->imageUrl = d->mCurrentPost->media;
+    }
+
+    setHeight();
 }
 
 void PostWidget::checkAnchor(const QUrl & url)
@@ -232,26 +219,33 @@ void PostWidget::initUi()
         connect(btnResend, SIGNAL(clicked(bool)), SLOT(slotResendPost()));
         baseText = &otherText;
     }*/
-    
-    d->mImage = "<img src=\"img://profileImage\" title=\""+ d->mCurrentPost->author.realName +"\" width=\"48\" height=\"48\" />";
+
+    d->mProfileImage = "<img src=\"img://profileImage\" title=\""+ d->mCurrentPost->author.realName +"\" width=\"48\" height=\"48\" />";
+    if(!d->imageUrl.isEmpty()) {
+      d->mImage = QString("<td width=\"%1\" height=\"%2\" style=\"padding-left: 5px; padding-left: 5px;\"><img src=\"img://postImage\"  /></td>").arg(d->mCurrentPost->mediaSizeWidth, d->mCurrentPost->mediaSizeHeight);
+    }
     d->mContent = prepareStatus(d->mCurrentPost->content);
     d->mSign = generateSign();
     setupAvatar();
+    fetchImage();
     setDirection();
     setUiStyle();
 
     d->mContent.replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive);
     d->mContent.replace("\n", "<br/>");
+
     d->mSign.replace("<a href","<a style=\"text-decoration:none\" href",Qt::CaseInsensitive);
 
     updateUi();
 }
 
-void PostWidget::updateUi() 
+void PostWidget::updateUi()
 {
-	_mainWidget->setHtml(baseText->arg( d->mImage, d->mContent,
+    _mainWidget->setHtml(baseText->arg( d->mProfileImage, d->mContent,
                                         d->mSign.arg(formatDateTime( d->mCurrentPost->creationDateTime )),
-                                        d->dir ));
+                                        d->dir,
+					d->mImage
+                                         ));
 }
 
 void PostWidget::setStyle(const QColor& color, const QColor& back, const QColor& read, const QColor& readBack, const QColor& own, const QColor& ownBack, const QFont& font)
@@ -365,7 +359,29 @@ void PostWidget::mousePressEvent(QMouseEvent* ev)
 
 void PostWidget::resizeEvent ( QResizeEvent * event )
 {
+    // only scale if image is present
+    if(!d->originalImage.isNull()) {
+	
+	// TODO: Find a way to calculate the difference we need to subtract.
+	int w = event->size().width() - 76;
+		
+	QPixmap newPixmap = d->originalImage.scaledToWidth(w, Qt::SmoothTransformation);
+	int newW = newPixmap.width();
+	int newH = newPixmap.height();
+	
+	// only use scaled image if it's smaller than the original one
+	if(newW <= d->originalImage.width() && newH <= d->originalImage.height()) {	// never scale up
+	    d->mImage = QString("<td width=\"%1\" height=\"%2\"><img src=\"img://postImage\"  /></td>").arg(newW, newH );
+	    QString url = "img://postImage";
+	    _mainWidget->document()->addResource( QTextDocument::ImageResource, url, newPixmap);
+	} else {
+	    d->mImage = QString("<td width=\"%1\" height=\"%2\"><img src=\"img://postImage\"  /></td>").arg(d->mCurrentPost->mediaSizeWidth, d->mCurrentPost->mediaSizeHeight);
+		QString url = "img://postImage";
+		_mainWidget->document()->addResource( QTextDocument::ImageResource, url, d->originalImage);
+	}
+    }
     setHeight();
+    updateUi();
     QWidget::resizeEvent(event);
 }
 
@@ -392,40 +408,20 @@ QString PostWidget::prepareStatus( const QString &txt )
     QString text = txt;
 //     text.replace( "&amp;", "&amp;amp;" );
     text = removeTags(text);
-    int pos = 0;
-    while(((pos = mUrlRegExp.indexIn(text, pos)) != -1)) {
-        QString link = mUrlRegExp.cap(0);
-        QString tmplink = link;
-        if ( (pos - 1 > -1 && ( text.at( pos - 1 ) != '@' && 
-             text.at( pos - 1 ) != '#' && text.at( pos - 1 ) != '!')) ||
-             pos == 0 ) { 
-        text.remove( pos, link.length() );
-        d->detectedUrls << link;
-        if ( !tmplink.startsWith(QLatin1String("http"), Qt::CaseInsensitive) &&
-             !tmplink.startsWith(QLatin1String("ftp"), Qt::CaseInsensitive) )
-             tmplink.prepend("http://");
-        static const QString hrefTemplate("<a href='%1' title='%1' target='_blank'>%2</a>");
-        tmplink = hrefTemplate.arg( tmplink, link );
-        text.insert( pos, tmplink );
+
+    d->detectedUrls = UrlUtils::detectUrls(text);
+    Q_FOREACH(const QString &url, d->detectedUrls) {
+        QString httpUrl(url);
+        if ( !httpUrl.startsWith(QLatin1String("http"), Qt::CaseInsensitive) &&
+             !httpUrl.startsWith(QLatin1String("ftp"), Qt::CaseInsensitive) ) {
+            httpUrl.prepend("http://");
+            text.replace(url, httpUrl);
         }
-        pos += tmplink.length();
+
+        text.replace(url, hrefTemplate.arg( httpUrl, url ));
     }
 
-    pos = 0;
-    while(((pos = mEmailRegExp.indexIn(text, pos)) != -1)) {
-        QString link = mEmailRegExp.cap(0);
-        QString tmplink = link;
-        if ( (pos - 1 > -1 && ( text.at( pos - 1 ) != '@' &&
-            text.at( pos - 1 ) != '#' && text.at( pos - 1 ) != '!')) ||
-            pos == 0 ) {
-            tmplink.prepend("mailto:");
-            text.remove( pos, link.length() );
-            static const QString hrefTemplate("<a href='%1' title='%1'>%2</a>");
-            tmplink = hrefTemplate.arg( tmplink, link );
-            text.insert( pos, tmplink );
-            }
-            pos += tmplink.length();
-    }
+    text = UrlUtils::detectEmails(text);
 
     if(AppearanceSettings::isEmoticonsEnabled())
         text = MediaManager::self()->parseEmoticons(text);
@@ -533,12 +529,26 @@ QString PostWidget::generateResendText()
     }
 }
 
+void PostWidget::fetchImage() {
+    if(d->imageUrl.isEmpty())
+        return;
+
+    QPixmap pix = MediaManager::self()->fetchImage(d->imageUrl, MediaManager::Async);
+
+    if(!pix.isNull())
+        slotImageFetched(d->imageUrl, pix);
+    else {
+        connect( MediaManager::self(), SIGNAL(imageFetched(QString,QPixmap)),
+                 this, SLOT(slotImageFetched(QString, QPixmap)));
+    }
+}
+
 void PostWidget::setupAvatar()
 {
-    QPixmap *pix = MediaManager::self()->fetchImage( d->mCurrentPost->author.profileImageUrl,
+   QPixmap pix = MediaManager::self()->fetchImage( d->mCurrentPost->author.profileImageUrl,
                                       MediaManager::Async );
-    if(pix)
-        avatarFetched(d->mCurrentPost->author.profileImageUrl, *pix);
+    if(!pix.isNull())
+        avatarFetched(d->mCurrentPost->author.profileImageUrl, pix);
     else {
         connect( MediaManager::self(), SIGNAL( imageFetched(QString,QPixmap)),
                 this, SLOT(avatarFetched(QString, QPixmap) ) );
@@ -572,6 +582,18 @@ void PostWidget::avatarFetchError(const QString& remoteUrl, const QString& errMs
     }
 }
 
+void PostWidget::slotImageFetched(const QString& remoteUrl, const QPixmap& pixmap) {
+
+    if(remoteUrl == d->imageUrl) {
+	QString url = "img://postImage";
+	QPixmap newPixmap = pixmap.scaled(d->mCurrentPost->mediaSizeWidth, d->mCurrentPost->mediaSizeHeight);
+        _mainWidget->document()->addResource( QTextDocument::ImageResource, url, newPixmap);
+	d->originalImage = pixmap;
+        updateUi();
+        disconnect( MediaManager::self(), SIGNAL(imageFetched(QString, QPixmap)), this, SLOT(slotImageFetched(QString, QPixmap)));
+    }
+}
+
 QMap<QString, KPushButton* >& PostWidget::buttons()
 {
     return d->mUiButtons;
@@ -592,12 +614,12 @@ void PostWidget::slotPostError(Account* theAccount, Choqok::Post* post,
 
 QString PostWidget::avatarText() const
 {
-    return d->mImage;
+    return d->mProfileImage;
 }
 
 void PostWidget::setAvatarText(const QString& text)
 {
-    d->mImage = text;
+    d->mProfileImage = text;
     updateUi();
 }
 
